package be.iminds.ilabt.jfed.restapi.json;

import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.util.jsonld.JsonLdObjectsMetaData;
import be.iminds.ilabt.util.jsonld.impl.PrimaryIdObjectBuilder;

import java.net.URI;
import java.net.URL;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

@JsonLdObjectsMetaData.JsonLdObjectBuilderInfo(objectClass = Experiment.class)
public class ExperimentBuilder extends PrimaryIdObjectBuilder<Long, Experiment, ExperimentBuilder> {
    private Timestamp expire;
    private Timestamp created;
    private String requestRspec;
    private String manifestRspec;
    private String speaksForCredential;
    private GeniUrn speakingFor;
    private String project;
    private String sliceName;
    private URL am;
    private List<GeniUrn> sharedWith;
    private GeniUrn owner;
    private GeniUrn sliceUrn;
    private Experiment.Status status;
    private Experiment.StatusDetails statusDetails;

    public ExperimentBuilder() {
    }

    public ExperimentBuilder(Experiment orig) {
        super(orig);
        this.expire = orig.getExpire();
        this.created = orig.getCreated();
        this.requestRspec = orig.getRequestRspec();
        this.manifestRspec = orig.getManifestRspec();
        this.speaksForCredential = orig.getSpeaksForCredential();
        this.speakingFor = orig.getSpeakingFor();
        this.project = orig.getProject();
        this.sliceName = orig.getSliceName();
        this.am = orig.getAm();
        this.sharedWith = orig.getSharedWith();
        this.owner = orig.getOwner();
        this.sliceUrn = orig.getSliceUrn();
        this.status = orig.getStatus();
        this.statusDetails = orig.getStatusDetails();
    }

    public ExperimentBuilder setExpire(Timestamp expire) {
        this.expire = expire;
        return this;
    }

    public ExperimentBuilder setCreated(Timestamp created) {
        this.created = created;
        return this;
    }

    public ExperimentBuilder setRequestRspec(String requestRspec) {
        this.requestRspec = requestRspec;
        return this;
    }

    public ExperimentBuilder setManifestRspec(String manifestRspec) {
        this.manifestRspec = manifestRspec;
        return this;
    }

    public ExperimentBuilder setSpeaksForCredential(String speaksForCredential) {
        this.speaksForCredential = speaksForCredential;
        return this;
    }

    public ExperimentBuilder setSpeakingFor(String speakingFor) throws GeniUrn.GeniUrnParseException {
        if (speakingFor == null) {
            this.speakingFor = null;
        } else {
            this.speakingFor = new GeniUrn(speakingFor);
        }
        return this;
    }
    public ExperimentBuilder setSpeakingFor(GeniUrn speakingFor) {
        this.speakingFor = speakingFor;
        return this;
    }

    public ExperimentBuilder setProject(String project) {
        this.project = project;
        return this;
    }

    public ExperimentBuilder setSliceName(String sliceName) {
        this.sliceName = sliceName;
        return this;
    }

    public ExperimentBuilder setAm(URL am) {
        this.am = am;
        return this;
    }

    public ExperimentBuilder setSharedWith(List<GeniUrn> sharedWith) {
        this.sharedWith = sharedWith;
        return this;
    }
    public ExperimentBuilder addSharedWith(GeniUrn sharedWithUser) {
        if (this.sharedWith == null) {
            this.sharedWith = new ArrayList<>();
        }
        this.sharedWith.add(sharedWithUser);
        return this;
    }

    public ExperimentBuilder setOwner(GeniUrn owner) {
        this.owner = owner;
        return this;
    }

    public ExperimentBuilder setSliceUrn(GeniUrn sliceUrn) {
        this.sliceUrn = sliceUrn;
        return this;
    }

    public ExperimentBuilder setStatus(Experiment.Status status) {
        this.status = status;
        return this;
    }

    public ExperimentBuilder setStatusDetails(Experiment.StatusDetails statusDetails) {
        this.statusDetails = statusDetails;
        return this;
    }

    public ExperimentBuilder setGlobalSliverStatus(String globalSliverStatus) {
        if (this.statusDetails == null) {
            this.statusDetails = new Experiment.StatusDetails(null, null, null, null);
        }
        Experiment.StatusDetails orig = this.statusDetails;
        this.statusDetails = new Experiment.StatusDetails(globalSliverStatus, orig.getAllocationStatus(), orig.getOperationalStatus(), orig.getjFedExperimentState());
        if (this.statusDetails.getAllocationStatus() == null &&
                this.statusDetails.getGlobalSliverStatus() == null &&
                this.statusDetails.getOperationalStatus() == null &&
                this.statusDetails.getjFedExperimentState() == null)
            this.statusDetails = null;
        return this;
    }

    public ExperimentBuilder setAllocationStatus(String allocationStatus) {
        if (this.statusDetails == null) {
            this.statusDetails = new Experiment.StatusDetails(null, null, null, null);
        }
        Experiment.StatusDetails orig = this.statusDetails;
        this.statusDetails = new Experiment.StatusDetails(orig.getGlobalSliverStatus(), allocationStatus, orig.getOperationalStatus(), orig.getjFedExperimentState());
        if (this.statusDetails.getAllocationStatus() == null &&
                this.statusDetails.getGlobalSliverStatus() == null &&
                this.statusDetails.getOperationalStatus() == null &&
                this.statusDetails.getjFedExperimentState() == null)
            this.statusDetails = null;
        return this;
    }

    public ExperimentBuilder setOperationalStatus(String operationalStatus) {
        if (this.statusDetails == null) {
            this.statusDetails = new Experiment.StatusDetails(null, null, null, null);
        }
        Experiment.StatusDetails orig = this.statusDetails;
        this.statusDetails = new Experiment.StatusDetails(orig.getGlobalSliverStatus(), orig.getAllocationStatus(), operationalStatus, orig.getjFedExperimentState());
        if (this.statusDetails.getAllocationStatus() == null &&
                this.statusDetails.getGlobalSliverStatus() == null &&
                this.statusDetails.getOperationalStatus() == null &&
                this.statusDetails.getjFedExperimentState() == null)
            this.statusDetails = null;
        return this;
    }

    public ExperimentBuilder setJFedExperimentState(String jfedExperimentState) {
        if (this.statusDetails == null) {
            this.statusDetails = new Experiment.StatusDetails(null, null, null, null);
        }
        Experiment.StatusDetails orig = this.statusDetails;
        this.statusDetails = new Experiment.StatusDetails(orig.getGlobalSliverStatus(), orig.getAllocationStatus(), orig.getOperationalStatus(), jfedExperimentState);
        if (this.statusDetails.getAllocationStatus() == null &&
                this.statusDetails.getGlobalSliverStatus() == null &&
                this.statusDetails.getOperationalStatus() == null &&
                this.statusDetails.getjFedExperimentState() == null)
            this.statusDetails = null;
        return this;
    }

    public Experiment create() {
        return new Experiment(id, /*name,*/ expire, created, requestRspec, manifestRspec, speaksForCredential, speakingFor, 
                project, sliceName, am, sharedWith, owner, sliceUrn, status, statusDetails, uri,
                mustSerializeAsEmbeddedObjectForNonMinimizedCreate());
    }

    @Override
    public Experiment createMinimized(JsonLdObjectsMetaData.Minimization minimization) {
        if (minimization == JsonLdObjectsMetaData.Minimization.FULL)
            return create(); //FULL must be treated differently to prevent recursion problems
        return new Experiment(minimization.includeId() ? id : null,
                minimization.includeOtherProperties() ? expire : null,
                minimization.includeOtherProperties() ? created : null,
                minimization.includeOtherProperties() ? requestRspec : null,
                minimization.includeOtherProperties() ? manifestRspec : null,
                minimization.includeOtherProperties() ? speaksForCredential : null ,
                minimization.includeOtherProperties() ? speakingFor : null,
                minimization.includeOtherProperties() ? project : null,
                minimization.includeOtherProperties() ? sliceName : null,
                minimization.includeOtherProperties() ? am : null,
                minimization.includeOtherProperties() ? sharedWith : null,
                minimization.includeOtherProperties() ? owner : null,
                minimization.includeOtherProperties() ? sliceUrn : null,
                minimization.includeOtherProperties() ? status : null,
                minimization.includeOtherProperties() ? statusDetails : null,
                minimization.includeId() ? uri : null,
                minimization.serializeAsEmbeddedObject());
    }
    public static List<Experiment> minimize(JsonLdObjectsMetaData.Minimization minimization, Collection<Experiment> in) {
        if (in == null) return null;
        List<Experiment> res = new ArrayList<>();
        for (Experiment tb : in) {
            res.add(new ExperimentBuilder(tb).createMinimized(minimization));
        }
        return res;
    }
    public static Experiment minimize(JsonLdObjectsMetaData.Minimization minimization, Experiment in) {
        if (in == null) return null;
        return new ExperimentBuilder(in).createMinimized(minimization);
    }

    public Timestamp getExpire() { return expire; }
    public Timestamp getCreated() { return created; }
    public String getRequestRspec() { return requestRspec; }
    public String getManifestRspec() { return manifestRspec; }
    public String getSpeaksForCredential() { return speaksForCredential; }
    public GeniUrn getSpeakingFor() { return speakingFor; }
    public String getProject() { return project; }
    public String getSliceName() { return sliceName; }
    public URL getAm() { return am; }
    public List<GeniUrn> getSharedWith() { return sharedWith; }
    public GeniUrn getOwner() { return owner; }
    public GeniUrn getSliceUrn() { return sliceUrn; }
    public Experiment.Status getStatus() { return status; }
    public Experiment.StatusDetails getStatusDetails() { return statusDetails; }
}